package com.tecnavia.firebase.crashlytics;

import android.content.Context;
import android.util.Log;

import com.google.firebase.crashlytics.FirebaseCrashlytics;

import static com.tecnavia.firebase.crashlytics.Constants.*;

import org.json.JSONException;
import org.json.JSONObject;

public class ReactNativeFirebaseCrashlyticsInitProvider extends ReactNativeFirebaseInitProvider {

	private static final String TAG = "RNFBCrashlyticsInit";

	private static boolean getKeyFromTecnaviaRc(String key, boolean defaultValue) {
		try {
			JSONObject obj = new JSONObject(BuildConfig.TECNAVIARC);
			if (obj.has(key)) {
				return obj.getBoolean(key);
			}
		} catch (JSONException e) {
			throw new RuntimeException(e);
		}
		return defaultValue;
	}

	static boolean isCrashlyticsCollectionEnabled(Context context) {
		ReactNativeFirebasePreferences prefs = ReactNativeFirebasePreferences.getSharedInstance();
		boolean enabled = prefs.getBooleanValue(context, AUTO_COLLECTION_ENABLED, getKeyFromTecnaviaRc(AUTO_COLLECTION_ENABLED, true));
		Log.d(TAG, "isCrashlyticsAutoCollectionEnabled: " + enabled);

		if (BuildConfig.DEBUG) {
			//in debug mode enable crashlytics just if it is forced by json setting
			if (!getKeyFromTecnaviaRc(AUTO_COLLECTION_ENABLED, false)) {
				enabled = false;
			}
		}
		return enabled;
	}

	static boolean isErrorGenerationOnJSCrashEnabled(Context context) {
		ReactNativeFirebasePreferences prefs = ReactNativeFirebasePreferences.getSharedInstance();
		boolean enabled = prefs.getBooleanValue(context, IS_ERROR_GENERATION_ON_JS_CRASH_ENABLED, getKeyFromTecnaviaRc(IS_ERROR_GENERATION_ON_JS_CRASH_ENABLED, true));
		Log.d(TAG, "isErrorGenerationOnJSCrashEnabled: " + enabled);
		return enabled;
	}

	static boolean isCrashlyticsJavascriptExceptionHandlerChainingEnabled(Context context) {
		ReactNativeFirebasePreferences prefs = ReactNativeFirebasePreferences.getSharedInstance();
		boolean enabled = prefs.getBooleanValue(context, JAVASCRIPT_EXCEPTION_HANDLER_CHAINING_ENABLED, getKeyFromTecnaviaRc(JAVASCRIPT_EXCEPTION_HANDLER_CHAINING_ENABLED, true));
		Log.d(TAG, "isCrashlyticsJavascriptExceptionHandlerChainingEnabled: " + enabled);
		return enabled;
	}

	@Override
	public boolean onCreate() {
		super.onCreate();

		try {
			FirebaseCrashlytics.getInstance().setCrashlyticsCollectionEnabled(isCrashlyticsCollectionEnabled(getContext().getApplicationContext()));
			Log.i(TAG, "initialization successful");
		} catch (Exception exception) {
			Log.e(TAG, "initialization failed", exception);
			return false;
		}

		return true;
	}
}
