#!/usr/bin/env bash
#
# Copyright (c) 2016-present Invertase Limited & Contributors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this library except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
set -e

_MAX_LOOKUPS=2;
_SEARCH_RESULT=''
_RN_ROOT_EXISTS=''
_CURRENT_LOOKUPS=1
_JSON_FILE_NAME='.tecnaviarc'
_CURRENT_SEARCH_DIR=${PROJECT_DIR}
_PLIST_BUDDY=/usr/libexec/PlistBuddy
_TARGET_PLIST="${BUILT_PRODUCTS_DIR}/${INFOPLIST_PATH}"
_DSYM_PLIST="${DWARF_DSYM_FOLDER_PATH}/${DWARF_DSYM_FILE_NAME}/Contents/Info.plist"

# plist arrays
_PLIST_ENTRY_KEYS=()
_PLIST_ENTRY_TYPES=()
_PLIST_ENTRY_VALUES=()

function setPlistValue {
  echo "info:      setting plist entry '$1' of type '$2' in file '$4'"
  ${_PLIST_BUDDY} -c "Add :$1 $2 '$3'" $4 || echo "info:      '$1' already exists"
}

function getFirebaseJsonKeyValue () {
  if [[ ${_RN_ROOT_EXISTS} ]]; then
    ruby -e "require 'rubygems';require 'json'; output=JSON.parse('$1'); puts output['$2']"
  else
    echo ""
  fi;
}

function jsonBoolToYesNo () {
  if [[ $1 == "false" ]]; then
    echo "NO"
  elif [[ $1 == "true" ]]; then
    echo "YES"
  else echo "NO"
  fi
}

echo "info: -> FirebaseCrashlytics build script started"
echo "info: 1) Locating ${_JSON_FILE_NAME} file:"

if [[ -z ${_CURRENT_SEARCH_DIR} ]]; then
  _CURRENT_SEARCH_DIR=$(pwd)
fi;

while true; do
  _CURRENT_SEARCH_DIR=$(dirname "$_CURRENT_SEARCH_DIR")
  if [[ "$_CURRENT_SEARCH_DIR" == "/" ]] || [[ ${_CURRENT_LOOKUPS} -gt ${_MAX_LOOKUPS} ]]; then break; fi;
  echo "info:      ($_CURRENT_LOOKUPS of $_MAX_LOOKUPS) Searching in '$_CURRENT_SEARCH_DIR' for a ${_JSON_FILE_NAME} file."
  _SEARCH_RESULT=$(find "$_CURRENT_SEARCH_DIR" -maxdepth 2 -name ${_JSON_FILE_NAME} -print | head -n 1)
  if [[ ${_SEARCH_RESULT} ]]; then
    echo "info:      ${_JSON_FILE_NAME} found at $_SEARCH_RESULT"
    break;
  fi;
  _CURRENT_LOOKUPS=$((_CURRENT_LOOKUPS+1))
done

if [[ ${_SEARCH_RESULT} ]]; then
  _JSON_OUTPUT_RAW=$(cat "${_SEARCH_RESULT}")
  _RN_ROOT_EXISTS=$(ruby -e "require 'rubygems';require 'json'; output=JSON.parse('$_JSON_OUTPUT_RAW'); puts output" || echo '')

  # config.crashlytics_debug_enabled
  _DEBUG_ENABLED=$(getFirebaseJsonKeyValue "$_JSON_OUTPUT_RAW" "crashlytics_debug_enabled")
  if [[ $_DEBUG_ENABLED ]]; then
    _PLIST_ENTRY_KEYS+=("crashlytics_debug_enabled")
    _PLIST_ENTRY_TYPES+=("bool")

    _DEBUG_ENABLED="YES"
    if [[ $_DEBUG_ENABLED == "false" ]]; then
        _DEBUG_ENABLED="NO"
    fi

    _PLIST_ENTRY_VALUES+=("$_DEBUG_ENABLED")
  fi

  # config.crashlytics_disable_auto_disabler - undocumented for now - mainly for debugging, document if becomes useful
  _CRASHLYTICS_AUTO_DISABLE_ENABLED=$(getFirebaseJsonKeyValue "$_JSON_OUTPUT_RAW" "crashlytics_disable_auto_disabler")
  if [[ $_CRASHLYTICS_AUTO_DISABLE_ENABLED == "true" ]]; then
    echo "Disabled Crashlytics auto disabler." # do nothing
  else
    _PLIST_ENTRY_KEYS+=("FirebaseCrashlyticsCollectionEnabled")
    _PLIST_ENTRY_TYPES+=("bool")
    _PLIST_ENTRY_VALUES+=("NO")
  fi

  # config.crashlytics_auto_collection_enabled
  _AUTO_COLLECTION_ENABLED=$(getFirebaseJsonKeyValue "$_JSON_OUTPUT_RAW" "crashlytics_auto_collection_enabled")
  if [[ $_AUTO_COLLECTION_ENABLED ]]; then
    _PLIST_ENTRY_KEYS+=("crashlytics_auto_collection_enabled")
    _PLIST_ENTRY_TYPES+=("bool")

    _AUTO_COLLECTION_ENABLED="YES"
    if [[ $_AUTO_COLLECTION_ENABLED == "false" ]]; then
        _AUTO_COLLECTION_ENABLED="NO"
    fi

    _PLIST_ENTRY_VALUES+=("$_AUTO_COLLECTION_ENABLED")
  fi

  # config.crashlytics_is_error_generation_on_js_crash_enabled
  _ERROR_ON_JS_CRASH_ENABLED=$(getFirebaseJsonKeyValue "$_JSON_OUTPUT_RAW" "crashlytics_is_error_generation_on_js_crash_enabled")
  if [[ $_ERROR_ON_JS_CRASH_ENABLED ]]; then
    _PLIST_ENTRY_KEYS+=("crashlytics_is_error_generation_on_js_crash_enabled")
    _PLIST_ENTRY_TYPES+=("bool")

    _ERROR_ON_JS_CRASH_ENABLED="YES"
    if [[ $_ERROR_ON_JS_CRASH_ENABLED == "false" ]]; then
        _ERROR_ON_JS_CRASH_ENABLED="NO"
    fi

    _PLIST_ENTRY_VALUES+=("$_ERROR_ON_JS_CRASH_ENABLED")
  fi

  # config.crashlytics_javascript_exception_handler_chaining_enabled
  _JS_EXCEPTION_HANDLER_CHAINING_ENABLED=$(getFirebaseJsonKeyValue "$_JSON_OUTPUT_RAW" "crashlytics_javascript_exception_handler_chaining_enabled")
  if [[ $_JS_EXCEPTION_HANDLER_CHAINING_ENABLED ]]; then
    _PLIST_ENTRY_KEYS+=("crashlytics_javascript_exception_handler_chaining_enabled")
    _PLIST_ENTRY_TYPES+=("bool")

    _JS_EXCEPTION_HANDLER_CHAINING_ENABLED="YES"
    if [[ $_JS_EXCEPTION_HANDLER_CHAINING_ENABLED == "false" ]]; then
        _JS_EXCEPTION_HANDLER_CHAINING_ENABLED="NO"
    fi

    _PLIST_ENTRY_VALUES+=("$_JS_EXCEPTION_HANDLER_CHAINING_ENABLED")
  fi
fi;

echo "info: 2) Injecting Info.plist entries: "

# Log out the keys we're adding
for i in "${!_PLIST_ENTRY_KEYS[@]}"; do
  echo "    ->  $i) ${_PLIST_ENTRY_KEYS[$i]}" "${_PLIST_ENTRY_TYPES[$i]}" "${_PLIST_ENTRY_VALUES[$i]}"
done

for plist in "${_TARGET_PLIST}" "${_DSYM_PLIST}" ; do
  if [[ -f "${plist}" ]]; then

    # paths with spaces break the call to setPlistValue. temporarily modify
    # the shell internal field separator variable (IFS), which normally
    # includes spaces, to consist only of line breaks
    oldifs=$IFS
    IFS="
"

    for i in "${!_PLIST_ENTRY_KEYS[@]}"; do
      setPlistValue "${_PLIST_ENTRY_KEYS[$i]}" "${_PLIST_ENTRY_TYPES[$i]}" "${_PLIST_ENTRY_VALUES[$i]}" "${plist}"
    done

    # restore the original internal field separator value
    IFS=$oldifs
  else
    echo "warning:   A Info.plist build output file was not found (${plist})"
  fi
done

if [[ ${PODS_ROOT} ]]; then
  echo "info: Exec FirebaseCrashlytics Run from Pods"
  "${PODS_ROOT}/FirebaseCrashlytics/run"
else
  echo "info: Exec FirebaseCrashlytics Run from framework"
  "${PROJECT_DIR}/FirebaseCrashlytics.framework/run"
fi

echo "info: <- FirebaseCrashlytics build script finished"